#ifndef LIBR2C2_GEOMETRY_H_
#define LIBR2C2_GEOMETRY_H_

#include <cmath>
#include <vector>
#include <msp/geometry/affinetransformation.h>
#include <msp/geometry/angle.h>
#include <msp/geometry/ray.h>
#include <msp/geometry/shape.h>
#include <msp/linal/vector.h>

namespace R2C2 {

typedef Msp::LinAl::Vector<float, 3> Vector;

inline float distance(const Vector &p, const Vector &q)
{ return (p-q).norm(); }

typedef Msp::Geometry::Angle<float> Angle;

typedef Msp::Geometry::AffineTransformation<float, 3> Transform;

inline Vector rotated_vector(const Vector &v, const Angle &a)
{ return Transform::rotation(a, Vector(0, 0, 1)).transform(v); }

inline Vector vector_at_angle(const Angle &a)
{ return rotated_vector(Vector(1, 0, 0), a); }

typedef Msp::Geometry::Shape<float, 3> Shape;
typedef Msp::Geometry::Ray<float, 3> Ray;
typedef Msp::Geometry::BoundingBox<float, 3> BoundingBox;

struct OrientedPoint
{
	Vector position;
	Angle rotation;
	Angle tilt;
};

} // namespace R2C2

#endif
