#ifndef LIBR2C2_TIMETABLE_H_
#define LIBR2C2_TIMETABLE_H_

#include <msp/datafile/objectloader.h>
#include "trackchain.h"
#include "trainai.h"

namespace R2C2 {

class Layout;

class Timetable: public TrainAI
{
public:
	class Loader: public Msp::DataFile::ObjectLoader<Timetable>
	{
	private:
		Layout &layout;

	public:
		Loader(Timetable &, Layout &);

	private:
		void row();
	};

	enum RowType
	{
		ARRIVE = 1,
		DEPART,
		THROUGH
	};

	struct Row
	{
		class Loader: public Msp::DataFile::ObjectLoader<Row>
		{
		private:
			Layout &layout;

		public:
			Loader(Row &, Layout &);

		private:
			void block(unsigned);
			void time(Msp::Time::RawTime);
			void zone(const std::string &, unsigned);
		};

		RowType type;
		TrackChain *target;
		TrackChain::Direction direction;
		Msp::Time::TimeDelta time;

		Row();

		void save(std::list<Msp::DataFile::Statement> &) const;
	};

	struct RowTypeMatch
	{
		RowType type;

		RowTypeMatch(RowType t): type(t) { }

		bool operator()(const Row &r) const { return r.type==type; }
	};

	sigc::signal<void, unsigned, const Row &> signal_row_added;
	sigc::signal<void, unsigned, const Row &> signal_row_modified;
	sigc::signal<void, unsigned> signal_row_removed;

private:
	std::list<Row> rows;
	std::list<Row>::iterator current_row;
	bool update_pending;
	bool sync_to_clock;

public:
	Timetable(Train &);

	void append_row(const Row &);
	void insert_row(unsigned, const Row &);
	void modify_row(unsigned, const Row &);
	void remove_row(unsigned);

	unsigned get_length() const { return rows.size(); }
	const Row &get_row(unsigned) const;

	virtual void tick(const Msp::Time::TimeDelta &);

	void save(std::list<Msp::DataFile::Statement> &) const;

private:
	void check_update(const std::list<Row>::const_iterator &);
	std::list<Row>::iterator find_trip(const std::list<Row>::iterator &, std::list<Row>::iterator *);
	void update_route();
	void event(TrainAI &, const Message &);
	void record_time();
};

} // namespace R2C2

#endif
